#include <fstream>
#include "constexpr.h"
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "borderDrawing.h"

BorderDrawing::BorderDrawing(std::ofstream& errorLog)
{
	loadTextureAndSprite(errorLog, "textures/elements/borderSquare.png", grayTexture, graySprite);
	loadTextureAndSprite(errorLog, "textures/elements/gameOverBorderSquare.png", redTexture, redSprite);
}

void BorderDrawing::loadTextureAndSprite(std::ofstream& errorLog, const std::string& texturePath, sf::Texture& texture, sf::Sprite& sprite)
{
	if( texture.loadFromFile(texturePath) )
	{
		sprite.setTexture(texture);
	}
	else{
		errorLog << "Error: couldn't load the texture file '" << texturePath << "' .\n";
	}
}

void BorderDrawing::drawAllBorders(sf::RenderWindow& rendWindow)
{
	drawWall(rendWindow, 0);
	drawWall(rendWindow, (BIGMAT_WIDTH + 1) * SQR_SIZE);
	drawBottomWall(rendWindow);
}

void BorderDrawing::drawWall(sf::RenderWindow& rendWindow, int wall_x_pos)
{
	int sprite_y_pos{0};
	for( unsigned y_pos{0} ; y_pos < BIGMAT_HEIGHT ; ++y_pos )
	{
		if( y_pos <= 2 )
		{
			redSprite.setPosition(wall_x_pos, sprite_y_pos);
			rendWindow.draw(redSprite);
		}
		else{
			graySprite.setPosition(wall_x_pos, sprite_y_pos);
			rendWindow.draw(graySprite);
		}
		sprite_y_pos += SQR_SIZE;
	}
}

void BorderDrawing::drawBottomWall(sf::RenderWindow& rendWindow)
{
	graySprite.setPosition(0, SCREENH-SQR_SIZE);
	for(unsigned x_pos{0} ; x_pos < BIGMAT_WIDTH + 2 ; ++x_pos )
	{
		rendWindow.draw(graySprite);
		graySprite.move(SQR_SIZE, 0);
	}
}