#include <string>
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "constexpr.h"
#include "highlightButton.h"

HighlightBtn::HighlightBtn(sf::Font& font, const std::string& text, const sf::Vector2f& centerPos):
	isClicked_{false},
	isMouseCursorOver_{false},
	text_{text, font},
	btnRect_{centerPos.x - HL_BTN_W / 2, centerPos.y - HL_BTN_H / 2, HL_BTN_W, HL_BTN_H}
{
	lightSquare_.setSize(sf::Vector2f{HL_BTN_W, HL_BTN_H});
	lightSquare_.setFillColor(sf::Color{255, 141, 0, 255});
	lightSquare_.setPosition(centerPos.x - HL_BTN_W / 2, centerPos.y - HL_BTN_H / 2);
	text_.setCharacterSize(24);
	text_.setFillColor(sf::Color::White);
	centerText(centerPos);
}
	
void HighlightBtn::centerText(const sf::Vector2f& centerPos)
{
	sf::FloatRect textRect = text_.getLocalBounds();
	text_.setOrigin(textRect.left + textRect.width / 2.0f, textRect.top + textRect.height / 2.0f);
	text_.setPosition(centerPos);
}

void HighlightBtn::getUserEvents(int mouse_pos_x, int mouse_pos_y, bool mouseButton)
{
	if( mouse_pos_x >= btnRect_.left 
		&& mouse_pos_x < btnRect_.left + btnRect_.width 
		&& mouse_pos_y >= btnRect_.top 
		&& mouse_pos_y < btnRect_.top + btnRect_.height )
	{
		isMouseCursorOver_ = true;
		if( mouseButton ){
			isClicked_ = true;
		}
		else{
			isClicked_ = false;
		}
	}
	else{
		isMouseCursorOver_ = false;
		isClicked_ = false;
	}
}

bool HighlightBtn::clicked() const
{
	return isClicked_;
}

void HighlightBtn::draw(sf::RenderWindow& render) const
{
	if( isMouseCursorOver_ ){
		render.draw(lightSquare_);
	}
	render.draw(text_);
}