#include <cassert>
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "userInput.h"

UserInput::UserInput():
	keysStates{false, false, false, false},
	escapeState{false},
	closeWindow{false},
	mouse_pos_x{-1},
	mouse_pos_y{-1},
	leftMouseButton{false}
{
	
}

void UserInput::updateEvents(sf::RenderWindow& renderer)
{
	while( renderer.pollEvent(Event) )
	{
		if( Event.type == sf::Event::Closed )
		{
			closeWindow = true;
		}
		else if( Event.type == sf::Event::KeyPressed )
		{
			updateKeyboard(Event.key.code, true);
		}
		else if( Event.type == sf::Event::KeyReleased )
		{
			updateKeyboard(Event.key.code, false);
		}
		else if( Event.type == sf::Event::MouseMoved )
		{
			mouseMoved();
		}
		else if( Event.type == sf::Event::MouseButtonPressed)
		{
			mouseButton(true);
		}
		else if( Event.type == sf::Event::MouseButtonReleased )
		{
			mouseButton(false);
		}
	}
}

void UserInput::updateKeyboard(sf::Keyboard::Key& key, bool valueToSet)
{
	if( key == sf::Keyboard::Escape )
		escapeState = valueToSet;
	else if( key == sf::Keyboard::Left )
		keysStates[KEY::MV_PIECE_LEFT] = valueToSet;
	else if( key == sf::Keyboard::Right )
		keysStates[KEY::MV_PIECE_RIGHT] = valueToSet;
	else if( key == sf::Keyboard::Down )
		keysStates[KEY::DOWN_FASTER] = valueToSet;
	else if( key == sf::Keyboard::X )
		keysStates[KEY::ROT_PIECE_CWISE] = valueToSet;
	else if( key == sf::Keyboard::W )
		keysStates[KEY::ROT_PIECE_ANTICWISE] = valueToSet;
}

bool UserInput::getKeyState(size_t index) const
{
	assert( index < keysStates.size() );
	return keysStates[index];
}

void UserInput::setKeyState(size_t index, bool newValue)
{
	assert( index < keysStates.size() );
	keysStates[index] = newValue;
}

void UserInput::mouseMoved()
{
	mouse_pos_x = Event.mouseMove.x;
	mouse_pos_y = Event.mouseMove.y;
}

void UserInput::mouseButton(bool valueToSet)
{
	if( Event.mouseButton.button == sf::Mouse::Button::Left )
		leftMouseButton = valueToSet;
}

int UserInput::getMousePosX() const
{
	return mouse_pos_x;
}

int UserInput::getMousePosY() const
{
	return mouse_pos_y;
}

bool UserInput::getMouseLeftButton() const
{
	return leftMouseButton;
}

void UserInput::setMouseLeftButtonFalse()
{
	leftMouseButton = false;
}

bool UserInput::getWindowClose() const
{
	return closeWindow;
}

void UserInput::setWindowCloseFalse()
{
	closeWindow = false;
}

bool UserInput::getEscapeState() const
{
	return escapeState;
}

void UserInput::setEscapeKeyFalse()
{
	escapeState = false;
}