#include <fstream>
#include "globalGameObject.h"
#include "moveCurrentPieceToMatrix.h"
#include "drawPiece.h"
#include "drawMatrix.h"


GameObject::GameObject(std::ofstream& errorLog, unsigned skillLevel):
	drawingFreq{10},
	pieceFallTime{1000},
	borderDraw{errorLog},
	sqrDraw{errorLog},
	allPieces{errorLog, skillLevel},
	matrix{BIGMAT_WIDTH, BIGMAT_HEIGHT},
	random{{Difference{0, static_cast<int>(allPieces.size()) - 1}}},
	currentPiece{random, allPieces},
	scanLinesInf{},
	playScore{errorLog}
{
	
}

Piece GameObject::getCurrentPiece() const
{
	return allPieces.getPiece(currentPiece.pieceIndex);
}

Piece& GameObject::getCurrentPiece()
{
	return allPieces.getPiece(currentPiece.pieceIndex);
}

bool GameObject::isSquarePresent(int x_sqr, int y_sqr) const
{
	return getCurrentPiece().getPresenceSquare(currentPiece.rotateIndex, x_sqr, y_sqr);
}

int GameObject::getCurrentPieceWidth() const
{
	return getCurrentPiece().width(currentPiece.rotateIndex);
}

int GameObject::getCurrentPieceHeight() const
{
	return getCurrentPiece().height(currentPiece.rotateIndex);
}

void GameObject::moveCurrentPieceDowner()
{
	currentPiece.posy++;
	scanLinesInf.canScanToRemoveLines = false;
	scanLinesInf.bottomLine = 0;
}

void GameObject::movePieceToMatrix()
{
	moveCurrentPieceToMatrix(currentPiece, getCurrentPiece(), matrix, scanLinesInf);
}

void GameObject::resetCurrentPiece()
{
	currentPiece.resetPieceValues(random, allPieces);
}

void GameObject::drawEverything(sf::RenderWindow& renderer)
{
	borderDraw.drawAllBorders(renderer);
	drawPiece(renderer, currentPiece, getCurrentPiece(), sqrDraw);
	drawMatrix(renderer, sqrDraw, matrix);
	playScore.drawScore(renderer);
}