#ifndef TEMP_MATRIX_2D
#define TEMP_MATRIX_2D

#include <vector>
#include <cassert>

struct SquareData
{
	unsigned color;
	bool filled;
	
	SquareData():
		color{0},
		filled{false}
	{}
};

template <typename T>

class MatrixTemp2D{
public:
    MatrixTemp2D(size_t width, size_t height):
        width_(width),height_(height),datas_(width*height)
	{}

    T operator()(size_t x, size_t y) const
	{
        assert(x < width_);
        assert(y < height_);
        return datas_[y*width_ + x];
    }
    T & operator()(size_t x, size_t y) 
	{
        assert(x < width_);
        assert(y < height_);
        return datas_[y*width_ + x];
    }
    T operator[](size_t i) const
	{
        assert( i < datas_.size() );
        return datas_[i];
    }
    T & operator[](size_t i)
	{
        assert( i < datas_.size() );
        return datas_[i];
    }

    void fillMatrix(const T& typeTemp)
	{
        std::fill( datas_.begin(), datas_.end(), typeTemp );
    }
	
    size_t size() const{
        return datas_.size();
    }
    size_t width() const{
        return width_;
    }
    size_t height() const{
        return height_;
    }
	
	bool isInsideBoundaries(size_t width, size_t height)
	{
		return width < width_ && height < height_;
	}
	
	bool isInsideBoundariesSigned(int width, int height)
	{
		return width < static_cast<int>(width_) && height < static_cast<int>(height_) && width >= 0 && height >= 0;
	}
	
	MatrixTemp2D(const MatrixTemp2D& toCopy):
		width_{toCopy.width_},
		height_{toCopy.height_},
		datas_(toCopy.width_ * toCopy.height_)
	{
		datas_ = toCopy.datas_;
	}
	
	void reverseDimensions()
	{
		size_t temp{width_};
		width_ = height_;
		height_ = temp;
	}

private:
    std::size_t width_;
    std::size_t height_;
    std::vector<T> datas_;
};


#endif //TEMP_MATRIX_2D
