#include <fstream>
#include <experimental/filesystem>
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "constexpr.h"
#include "playerScore.h"
#include "prefPathWithAPI.h"
#include "scoreGraphics.h"
#include "timeDelayClass.h"
#include "userInput.h"
#include "bestScores.h"

namespace fs = std::experimental::filesystem;

void bestScores::bestScoresContext(std::ofstream& errorLog, const PrefPathFinder& prefPath, sf::RenderWindow& renderer, UserInput& input, const PlayerScore& playerScore, unsigned skill)
{
	bestScores::createScoresFilesIfMissing(errorLog, prefPath);
	bool quitScoreContext{false};
	sf::Font arial;
	arial.loadFromFile(ARIALFONTPATH);
	ScoresGraphics scores{errorLog, prefPath, arial};
	TimeDelay drawDelay{20};
	scores.addCurrentScoreToBest(playerScore.getScore(), skill);
	
	bestScores::mainLoop(renderer, input, drawDelay, scores, quitScoreContext);
	
	scores.outputScoresToFile(errorLog, prefPath);
}

void bestScores::mainLoop(sf::RenderWindow& renderer, UserInput& input, TimeDelay& drawDelay, const ScoresGraphics& scores, bool& quitContext)
{
	while( !quitContext )
	{
		input.updateEvents(renderer);
		bestScores::quitWInp(input, quitContext);
		bestScores::drawAll(renderer, scores, drawDelay);
	}
}

void bestScores::quitWInp(UserInput& input, bool& quitContext)
{
	if( input.getWindowClose() || input.getEscapeState() )
		quitContext = true;
}

void bestScores::drawAll(sf::RenderWindow& renderer, const ScoresGraphics& scores, TimeDelay& drawDelay)
{
	if( drawDelay.delaySufficient(20) )
	{
		renderer.clear(sf::Color::Black);
		scores.drawScores(renderer);
		renderer.display();
	}
}

void bestScores::createScoresFilesIfMissing(std::ofstream& errorLog, const PrefPathFinder& prefPath)
{
	bestScores::createSingleScoreFile(errorLog, prefPath.getPath() + EASY_FILE_NAME);
	bestScores::createSingleScoreFile(errorLog, prefPath.getPath() + INTERM_FILE_NAME);
	bestScores::createSingleScoreFile(errorLog, prefPath.getPath() + HARD_FILE_NAME);
}

void bestScores::createSingleScoreFile(std::ofstream& errorLog, const std::string& fileFullPath)
{
	if( ! fs::exists(fileFullPath) )
	{
		if( std::ofstream scoreFile{fileFullPath} )
		{
			errorLog << "Create new file: " << fileFullPath << "  for the player score of tetris.\n";
			scoreFile << 0 << " " << 0 << " " << 0 << " " << 0 << " " << 0 << "\n"; 
		}
		else
		{
			errorLog << "Error: couldn't create '" << fileFullPath << "' file for SFML tetris scores.\n";
		}
	}
}